/* SpotWelder Steuerung
 * Harald Sattler, 2019-06-29
 *
 * Der Attiny85 im DigiSpark laeuft mit 16,5 Mhz (wenn der Digispark im Default-Modus laeuft).
 * Hier (ab V3.0) wird der AtTiny als AtTiny85 betrieben und programmiert, dann laeuft er mit 16 MHz.
 *
 * Basisplatine mit Anschluessen fuer 
 * - Fusspedal zum Ausloesen des Schweissimpulses
 * - Spannungsversorgung (DC, max. 14V. Der Spanungsregler des DigiSpark versorgt auch den Rest der Schaltung)
 * - SolidState Relais das den Mikrowellen-Trafo schaltet
 * - Stecksockel fuer den DigiSpark MicroController
 * - Stecksockel fuer das OLED Display (1,6" Diagonale, 128 x 64 Pixel)
 * Wichtig!
 * Der hohe Stromimpuls des Trafo bewirkt, dass das Einschaltsignal vom Microcontroller an das SolidState Relais fruehzeitig
 *   abgebrochen wird, die eingestellte Zeit wird also nicht erreicht.
 *   Daher ist auf der Basisplatine ein Stuetzkondensator platziert, der diesen Effekt unterdrueckt.
 *
 * Mit einem Spindelpoti wird die Impulsdauer eingestellt.
 * Ein Microtaster parallel zum Anschluss fuer den Fussschalter loest den Impuls ebenfalls aus (Test).
 * Der Impuls wird nur einmal ausgeloest, der Taster/Fussschalter muss losgelassen werden, bevor erneut ausgeloest werden kann.
 *
 * Der Taster/Fusspedal benoetigt einen externen PullUp mit maximal 1,5 kOhm, da am selben Pin die Beschaltung fuer den USB haengt.
 *   Die Z-Diode zieht ueber einen 68 Ohm Widerstand nach Masse. Mit PullUp von 1,5 kOhm erreicht der Pin einen Ruhepegel von
 *   ~3,2 V, was als High erkannt wird.
 * 
 * Mit dem Poti wird die Impulsdauer eingestellt und auf dem OLED-Display angezeigt.
 * Das Display wird nur aktualisiert, wenn sich die Einstellung des Poti geaendert hat.
 * Mit dem Taster/Fusspedal wird der Impuls ausgeloest.
 * Am Ausgang "Relais" wird ein SolidState Relais angesteuert, das den umgebauten Mikrowellen-Trafo primaerseitig fuer die 
 * Dauer des Impulses einschaltet. Moechte man ein normales Relais verwenden, muss eine passende Treiberstufe zwischengeschaltet
 * werden.
 * 
 * Das Display zeigt die minimal und maximal einstellbare sowie die aktuell eingestellte Impulsdauer an.
 * Fuer die Dauer des Impulses wird - als visuelle Rueckmeldung - das Display invers geschaltet (bei (zu) kurzer Impulsdauer wird 
 *   nur ein schmaler Balken invers dargestellt).
 *
 * 2019-07-12, V2.0
 * - Implementierung eines Menues zur Einstellung der Impulsanzahl
 *   Wird beim Einschalten (und nach Ablauf der Bootloader-Zeit) der Taster gedrueckt gehalten, wird direkt das Einstellungsmenue aufgerufen.
 *   Im laufenden Betrieb kann durch Festhalten des Taster fuer mehr als 1 Sekunde ebenfalls das Menue aufgerufen werden.
 *   Durch drehen am Poti wird der Wert geaendert.
 *   Nach dem Verlassen des Menues die Impulsdauer kontrollieren, da das Poti im Menue ggf. verstellt wurde.
 * - Speicherung der Einstellungen im EEPROM - (noch?) nicht implementiert, kein Platz mehr.
 * - Implementierung von Mehrfachimpulsen - die zur Laufzeit (nach jedem Start) eingestellte Anzahl von Impulsen wird ausgegeben.
 *   Nach Reboot ist die Anzahl immer 1.
 *
 * 2019-07-13, V3.0
 * - Umstellung von DigiSpark auf AtTiny85 - damit mehr Flash (8 statt 6 kB) weil der Bootloader wegfaellt.
 *   Drawback: Programmierung muss mit Programmer (z.B. mySmartUSB light) durchgefuehrt werden.
 *   Aber auch mit Micronucleus musste der Chip immer auf den USB umgesteckt werden, fuer mich also kein Nachteil.
 *   Die Umstellung bedingt die Verwendung anderer Libraries fuer I2C und OLED.
 *
 * 2019-07-18, V3.0
 * - Neue Erkenntnisse in Sachen Punktschweissen: 2 Impulse mit 180 ms bei ca. 1,4 m Drahtlaenge funktionieren meist ganz gut mit den neuen Spitzen.
 *   Die Spitzen sind 10er Kupferrundmaterial, 50 mm lang, innen ausgebohrt auf 7,5 mm. Da passt der 35qmm Draht saugend rein und wird mit vier Krner-
 *   Schlgen festgecrimpt. Dennoch werden die Spitzen beim Schweissen warm :(
 *   Die vorderen Enden der Spitzen sind im Winkel von 10 Grad bis auf ca. 2 mm Durchmesser abgedreht, die Schraege ist somit ca. 25 mm lang.
 * - Mit der neuen LIB hat das Display 8 Zeilen (Y=0..7). Die X-Koordinate ist irgendwie daneben, Positionieren funktioniert nicht -> st zu eruieren.
 *
 * 2019-07-21, V3.0
 * - Keine X-Positionierung verwendet, work around per Ausgabe von Spaces.
 * - Die im Menue durch Drehen am Poti eingestellte Anzahl von Impulsen wird im EEPROM gespeichert und beim Start von dort eingelesen.
 * - Die Impulsdauer wird ebenfalls im EEPROM abgelegt und auch wieder eingelesen, aber zur Laufzeit durch die Einstellung des Poti ueberschrieben.
 *   Dieser Teil der EEPROM-Speicherung ist also ueberfluessig, Vorbereitung auf spaetere Erweiterung (falls notwendig).
 *
 *   -->> Nach Bedienung des Poti im Menue zur Werteaenderung und Verlassen des Menues muss die Impulsdauer am Poti wieder korrekt eingestellt werden! <<--
 */
 
// includes
#include "SSD1306_minimal.h"
#include <EEPROM.h>

 // defines fuer die Pins
 #define TASTER 4
 #define POTI 	3
 #define RELAIS 1
 #define RESET 5										// nicht verwenden, der DigiSpark wird angehalten, wenn an Pin 5 eine LED (oder ein PullUp!) angeschlossen ist
 
 // definitions IIC & OLED
#define PIN_I2C_SDA 0									// TX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define PIN_I2C_SCL 2									// RX-Pin - die Pins koennen nicht geaendert werden!! Ist in der Lib so festgelegt
#define OLED_address  0x3c							    // I2C-Adresse des OLED Display

// defines fuer EEPROM Zugriff
#define EEPARAMS 100									// Startadresse fuer Parameterspeicher

// globale Variablen
 unsigned int potiWert;									// per analogRead() gelesener Wert des Poti
 unsigned int weldingTime;								// Wert des Impulses in [ms], Variable im Programm
 unsigned int impulsCount;								// Anzahl Schweiss-Impulse, Variable im Programm
 //unsigned long weldingMicros;
 unsigned long lastWeldingTime = 0;						// Merker fuer ScreenUpdate (nur aktualisieren, wenn Aenderung des Wertes)
 bool tasterSet = false;								// Flag fuer "ist der Taster gedrueckt?"
 unsigned long tasteLang = 0;							// Merker fuer langen Tastendruck

 // Puffer fuer OLED String-Ausgabe
char OLEDBuffer[8];

const unsigned int minImpulsAllowed = 10;				// minimale Impulsdauer
const unsigned int maxImpulsAllowed = 300;				// maximale Impulsdauer
const unsigned int maxImpulsAnzahl = 5;					// maximale Impulsanzahl

const unsigned int impulsDauer = 180;					// Impulsdauer, Konstante als Default, kann per Poti geandert werden
const unsigned int impulsAnzahl = 2;					// Impulsanzahl, Konstante als Default, kann per Menue geandert werden

bool inEinstellungen = false;							// Flag "wir sind im Einstellungen Menue"
unsigned char menueState = 0;							// welcher Parameter soll geaendert werden

SSD1306_Mini oled;										// OLED Objekt anlegen

// Potistellung erfassen und Wert in [ms] zurueckgeben
void readPoti()
{
	// Poti auslesen, ergibt Wert 0..1023
	potiWert = analogRead(POTI);

	// daraus Zeit in ms in weldingTime ablegen
	weldingTime = map(potiWert, 0, 1023, minImpulsAllowed, maxImpulsAllowed);	// eingrenzen auf erlaubte min/max Werte
	//weldingMicros = weldingTime * 1000;					// Probleme zur  Laufzeit mit delay(), daher delayMicroseconds() verwendet
}

// geaenderte Einstellungen im EEPROM persistent machen
void saveToEEPROM()
{
	EEPROM.put(EEPARAMS + 0, weldingTime);					// put und update schreibt nur dann, wenn der neue Wert unterschiedlich zum Gespeicherten ist
	EEPROM.put(EEPARAMS + sizeof(int), impulsCount);
}

void Einstellungen()
{
	// Ausgabe des Menues
	oled.clear();
	oled.cursorTo(0, 2);									// Zeile 2
	oled.printString("Einstellungen");						

	oled.cursorTo(0, 7);									// Zeile 7
	oled.printString("Impulse:");
	//oled.printString("Dauer [ms]:", 4, 0);

	menueState = 1;											// initial immer Impulsanzahl auswaehlen

	// Poti abfragen und Wert merken fuer Vergleich auf Aenderung
	unsigned int potiNav = analogRead(POTI);
	unsigned int potiNavWert = 0;

	while (inEinstellungen == true)
	{
		// Abfrage des Tasters zur Navigation im Menue
		if ((digitalRead(TASTER) == 0) && (tasterSet == false))	// Taster (nach offen) gedrueckt
		{
			// feststellen ob Taste lange (> 1s) gedrueckt wird
			tasteLang = millis();
			tasterSet = true;

			// Behandlung der Tastendruecke
			//if (menueState == 1)							// Impulsanzahl wird eingestellt
			//{
			//	menueState = 2;
			//}
			//else 
			//{
			//	menueState = 1;
			//}
		}
		else if ((tasterSet == true) && (digitalRead(TASTER) == 1))	// Taster wieder losgelassen
		{
			tasteLang = millis();								// Status "Taste lange gedrueckt" loeschen
			tasterSet = false;
		}
		else if ((digitalRead(TASTER) == 0) && (tasterSet == true))	// Taster festgehalten
		{
			if (millis() > tasteLang + 1000)					// Taste laenger als 1 Sekunde gedrueckt
			{
				tasteLang = millis();
				inEinstellungen = false;						// Einstellungen verlassen
				oled.clear();
				lastWeldingTime = 0;							// Screen refresh erzwingen

				saveToEEPROM();									// Speicherung der geaenderten Einstellungen im EEPROM
			}
			delay(100);											// bei gedrueckter Taste Prellen unterdruecken
		}

		// Markierung des aenderbaren Parameters mit "x" in letzter Spalte
		//sendStrXY("x", 2 * menueState, 15);
		// Loeschen des jeweils anderen "x"
		//sendStrXY(" ", 6 - (2 * menueState), 15);

		if (inEinstellungen == true)								// Einstellungen vornehmen
		{			
			potiNavWert = analogRead(POTI);							// Poti fuer Vergleich einlesen

			// Abfrage des Poti zum einstellen der Parameter
			if ((potiNav - potiNavWert) > 20)						// Parameter erhoehen
			{
				//if (menueState == 1)
				//{
				impulsCount++;
				//}
				//else
				//{
				//	impulsDauer++;
				//}
				potiNav = potiNavWert;
			}

			if ((potiNavWert - potiNav) > 20)						// Parameter verringern
			{
				//if (menueState == 1)
				//{
				impulsCount--;
				//}
				//else
				//{
				//	impulsDauer--;
				//}
				potiNav = potiNavWert;
			}

			// Parameter auf zulaessige Werte beschraenken
			impulsCount = constrain(impulsCount, 1, 5);
			//impulsDauer = constrain(impulsDauer, minImpulsAllowed, maxImpulsAllowed);

			// Parameter anzeigen
			dtostrf(impulsCount, 2, 0, OLEDBuffer);					// Impuls Anzahl in String wandeln
			oled.cursorTo(55, 7);									// Zeile 7
			oled.printString(OLEDBuffer);
			//dtostrf(impulsDauer, 2, 0, OLEDBuffer);				// Impuls Dauer in String wandeln
			//sendStrXY(OLEDBuffer, 4, 11);


		}
		// debug: Status "Einstellungen"
		//sendStrXY("e", 0, 0);
	}
}

void setup()
{
	// Pins initialisieren
	pinMode(TASTER, INPUT_PULLUP);
	pinMode(RELAIS, OUTPUT);
	pinMode(RESET, INPUT);

	// Initialisierung IIC OLED Dispaly	
	// Serial.println("Initialise OLED screen...");
	oled.init(0x3C);

	// Taster gedrueckt? -> Ja, dann ins Einstellungen-Menue
	if (digitalRead(TASTER) == 0)
	{
		inEinstellungen = true;								// Flag setzen und ...
		Einstellungen();									// ...Einstellungen aufrufen
	}

	// EEPROM mit Defaults vorbelegen, falls die Werte noch nicht gesetzt wurden
	EEPROM.get(EEPARAMS, weldingTime);
	if(weldingTime > maxImpulsAllowed)	EEPROM.put(EEPARAMS, impulsDauer);
	EEPROM.get(EEPARAMS + sizeof(int), impulsCount);
	if(impulsCount > maxImpulsAnzahl)	EEPROM.put(EEPARAMS + sizeof(int), impulsAnzahl);

	// EEPROM Werte einlesen
	EEPROM.get(EEPARAMS, weldingTime);						// Impulsdauer einlesen
	EEPROM.get(EEPARAMS + sizeof(int), impulsCount);		// Impulsanzahl einlesen

	// Test
	//oled.clear();
	//oled.cursorTo(0, 0);
	//oled.printString("Zeile 0");							// Zeile 0
	//oled.cursorTo(0, 1);
	//oled.printString("Zeile 1");							// Zeile 0
	//oled.cursorTo(0, 2);
	//oled.printString("Zeile 2");							// Zeile 2
	//oled.cursorTo(0, 3);
	//oled.printString("Zeile 3");							// Zeile 4
	//oled.cursorTo(0, 4);
	//oled.printString("Zeile 4");							// Zeile 4
	//oled.cursorTo(0, 5);
	//oled.printString("Zeile 5");							// Zeile 4
	//oled.cursorTo(0, 6);
	//oled.printString("Zeile 6");							// Zeile 4
	//oled.cursorTo(0, 7);
	//oled.printString("Zeile 7");							// Zeile 4
	//while (1);	// hier anhalten

	// Ausgabe des Splashscreen
	oled.clear();
	oled.cursorTo(0, 1); 
	oled.printString("SpotWelder");						
	oled.cursorTo(0, 3); 
	oled.printString("V3.0 - 07/2019");
	oled.cursorTo(0, 5); 
	oled.printString("Harald Sattler");
	delay(2000);
	oled.clear();
}

void loop()
{
	static unsigned long looptime = 0;

	// debug: Status "loop"
	//sendStrXY("l", 0, 0);

	// Ausgabe auf OLED, jede Sekunde aktualisiert
	if (millis() > looptime + 500) 							// jede halbe Sekunde aktualisieren
	{	
		readPoti();											// liest Poti und liefert Impulsdauer in ms
		
		if (lastWeldingTime != weldingTime)					// hat sich der Wert geaendert? (Poti gedreht, Drift, ...)
		{
			// geaenderte Impulszeit merken
			lastWeldingTime = weldingTime; 

			// dann Display aktualisieren
			oled.cursorTo((2 + 1) * 6, 1);
			oled.printString("SpotWelder");							

			// Anzahl Impulse ausgeben
			oled.cursorTo(0, 3);
			oled.printString("Impulse:");
			oled.printString(" ");
			dtostrf(impulsCount, 2, 0, OLEDBuffer);		// minimale Impulsdauer in String wandeln
			oled.printString(OLEDBuffer);

			// Impulsdauer-Grenzen ausgeben
			oled.cursorTo(0, 5);							// Zeile 5
			oled.printString("Dauer: ");

			// minImpulsAllowed als Int in den OLEDBuffer kopieren
			dtostrf(minImpulsAllowed, 2, 0, OLEDBuffer);	// minimale Impulsdauer in String wandeln
			oled.printString(OLEDBuffer);

			oled.printString("..");

			// maxImpulsAllowed als Int in den OLEDBuffer kopieren
			maxImpulsAllowed < 100 ? dtostrf(maxImpulsAllowed, 2, 0, OLEDBuffer) : dtostrf(maxImpulsAllowed, 3, 0, OLEDBuffer);	// maxImpuls Dauer in String wandeln
			oled.printString(OLEDBuffer);

			oled.cursorTo(0, 7);							// Zeile 7
			//oled.printString("Impuls:    ms");
			oled.printString("Impuls: ");

			// weldingTime als Int in den OLEDBuffer kopieren
			weldingTime < 100 ? oled.printString(" ") : oled.printString("");	// Hunderterstelle als Blank bei Werten bis 99
			weldingTime < 100 ? dtostrf(weldingTime, 2, 0, OLEDBuffer) : dtostrf(weldingTime, 3, 0, OLEDBuffer);
			oled.printString(OLEDBuffer);
		}
		looptime = millis();
	}
	
	// Taster abfragen und Impuls ausloesen (Taster ist aktiv LOW)
	if((digitalRead(TASTER) == 0) && (tasterSet == false))		// Taster (nach offen) gedrueckt
	{
		tasteLang = millis();									// feststellen ob Taste lange (> 1s) gedrueckt wird

		for (char counter = 0; counter < impulsCount;  counter++) // Anzahl parametrierter Impulse ausgeben
		{
			oled.sendCommand(0xa7);								// Set Inverse Display (visuelle Rueckmeldung)

			// Set RelaisPin 
			digitalWrite(RELAIS, 1);							// Impuls einschalten
			delay(weldingTime);									// Impulsdauer abwarten
			digitalWrite(RELAIS, 0);							// Impuls wieder ausschalten

			oled.sendCommand(0xa6);								// Set Normal Display (not inverse) (visuelle Rueckmeldung)
			delay(200);											// kurz warten bevor ein neuer Impuls ausgeloest werden kann (Tastenprellen)
		}

		tasterSet = true;
	}
	else if((tasterSet == true) && (digitalRead(TASTER) == 1))	// Taster wieder losgelassen
	{
		tasteLang = millis();									// Status "Taste lange gedrueckt" loeschen
		tasterSet = false;
	}
	else if ((digitalRead(TASTER) == 0) && (tasterSet == true))	// Taster festgehalten
	{
		if (millis() > tasteLang + 1000)						// Taste laenger als 1 Sekunde gedrueckt
		{
			tasterSet = false;									// Status "Taste gedrueckt" loeschen
			tasteLang = millis();
			inEinstellungen = true;								// Flag setzen und ...
			Einstellungen();									// ...Einstellungen aufrufen
		}
		delay(100);												// bei gedrueckter Taste Prellen unterdruecken
	}
}